# fc_helper.py
#
# This file is part of python wrapper for the BROADWAY2 package.
#
# MIT License

# Copyright (c) 2013-2020 FibreCode GmbH <info@fibrecode.com>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


# helper functions to map/get sub-instances bounmd to different adapters

from broadway2 import broadway_api, smi_access, spi_access, phy_access, sjax_access


def get_phy_access(fca, phyAddr=0):
    """return correct phy instancce, if supported by this device
    """
    if not fca.is_connected():
        return None

    phyAccess = None  # Default, no phy_access

    if fca.DevName == 'FC401':
        smi = smi_access.SmiAccess(fca.Device)
        phyDict = smi.detect2()
        try:
            phyInst = phyDict[phyAddr]
        except:
            phyInst = {}

        if phyInst != {}:
            phyAccess = phy_access.PhyAccess(fca.Device, phyAddr, phyInst['Clause'])
    elif fca.DevName == 'FC602':
        phyAccess = phy_access.PhyAccess(fca.Device, 0, 1)
    elif fca.DevName == 'FC611':
        phyAccess = phy_access.PhyAccess(fca.Device, 0, 1)
    elif fca.DevName == 'FC612':
        phyAccess = phy_access.PhyAccess(fca.Device, 0, 1)
    elif fca.DevName == 'FC621':
        phyAccess = phy_access.PhyAccess(fca.Device, 0, 2)
    elif fca.DevName == 'FC631':
        phyAccess = phy_access.PhyAccess(fca.Device, 0, 2)
    else:
        phyAccess = phy_access.PhyAccess(fca.Device, 0)


    return phyAccess


def get_spi_access(fca, protocolId=0, speed=1e6):
    """return correct SJA11xx instance, if supported by this device
    
    protocolId for correct CPOL/CPHA. SJA1105x = 0, SJA1110x = 1
    speed: SPI Transaction clock, default 1MHz

    default will configure SJA1105 and speed of 1MHz
    """
    if not fca.is_connected():
        return None

    spiAccess = None

    if fca.DevName == 'FC401':
        spiAccess = spi_access.SpiAccess(fca.Device)
        
        spiAccess.set_protocol(protocolId)
        spiAccess.set_speed(speed)

    return spiAccess


def get_switch_access(spiAccess, cs=0):
    """return correct SJA11xx instance, if supported by this device
    
    protocolId for correct CPOL/CPHA. SJA1105x = 0, SJA1110x = 1
    cs: chip-select (0 or 1)
    speed: SPI Transaction clock, default 1MHz

    default will configure cs=0, SJA1105 and speed of 1MHz
    """

     # now we can generate two SJA-Access instances to each switch using CS
    sjaXAccess = sjax_access.SJAXAccess(spiAccess, cs)

    return sjaXAccess
