#!python3
# multi_raw_sender.py
#
# This file is part of python wrapper for the BROADWAY2 package.
#
# MIT License

# Copyright (c) 2013-2018 FibreCode GmbH <info@fibrecode.com>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import csv
import binascii

from broadway2 import broadway_api

class MultiRawSender:
    def __init__(self, delimiter=','):
        self.devTx = []
        self.lutDevName = {}    # lut to get name for given device.value
        self.lutNameDev = {}    # lut to get dev from name
        self.lutNameMACAddress = {}
        self.lutDevInst = {}
        self.delimiter = delimiter
        self.Pos = -1           # will be set to 0 after csv is verified

    def add(self, adapter, devTx):
        self.devTx.append(devTx)
        self.lutDevName[devTx.value] = adapter.FriendlyName
        self.lutNameDev[adapter.FriendlyName] = devTx
        self.lutNameMACAddress[adapter.FriendlyName] = adapter.MACAddress
        self.lutDevInst[devTx.value] = adapter.Inst

    def enumerate(self):
        for dev in self.devTx:
            yield dev


    def send_raw_frames(self, devName, srcMACAddr, frameList):
        """sendout given frames as array of ascii-hex formated frames
        devName is friendlyName mapped device
        srcMACAddr is option, if '' the adapter MAC-Address is used
        frameList as tuples (TgtMAC, EthType, Data)
        """
        sendCnt = len(frameList)
        try:
            dev = self.lutNameDev[devName]
        except:
            return []

        result = []
        for frame in frameList:
            if len(srcMACAddr) == 0:
                srcMAC = self.lutNameMACAddress[devName]
            else:
                srcMAC = srcMACAddr
                    
            sendData = frame[0]+srcMAC+frame[1]+frame[2]
            binData = binascii.a2b_hex(sendData)

            res = broadway_api.wait_for_tx(dev, 1000)
            if res: 
                frameId = broadway_api.submit_raw_frame(dev, binData)
                res = broadway_api.send_raw_frames(dev)
                
                res = broadway_api.wait_for_raw_tx_completion(dev, frameId, 1000)
                timeStamp = broadway_api.get_raw_tx_timestamp(dev, frameId)
                
                result.append((frameId, timeStamp[0], timeStamp[1], srcMAC))

        return result


    def verify_csv(self, f):
        """verify if given csv-file matches expected header file
        Header: ['Name', 'TgtAddr', 'SrcAddr', 'EthType', 'Data']:

        returns True, if syntax and used devices are found
        """
        reader = csv.reader(f, delimiter=self.delimiter)
        head = next(reader)
        if head == ['Name', 'TgtAddr', 'SrcAddr', 'EthType', 'Data']:
            errCnt = 0
            for row in reader:
                if len(row) == 5:
                    try:
                        test = self.lutNameDev[row[0]]
                    except:
                        errCnt += 1
        else:
            errCnt = 99

        if errCnt == 0:
            self.Pos = 0
        return errCnt


    def step(self, f):
        if self.Pos == 0:   # start at beginning
            f.seek(0)
            self.csvReader = csv.reader(f, delimiter=self.delimiter)
            head = next(self.csvReader)
        try:
            data = next(self.csvReader)
            if len(data) == 5:
                
                dev = self.lutNameDev[data[0]]
                
                if len(data[2]) == 0:
                    srcMAC = self.lutNameMACAddress[data[0]]
                else:
                    srcMAC = data[2]
                    
                sendData = data[1]+srcMAC+data[3]+data[4]
                binData = binascii.a2b_hex(sendData)

                res = broadway_api.wait_for_tx(dev, 1000)
                if not res:
                    print('waitfor tx failed!')

                frameId = broadway_api.submit_raw_frame(dev, binData)

                res = broadway_api.send_raw_frames(dev)

                res = broadway_api.wait_for_raw_tx_completion(dev, frameId, 1000)

        except:
            return False
        self.Pos += 1

        return True
