# raw_frame.py
#
# This file is part of python wrapper for the BROADWAY2 package.
#
# MIT License

# Copyright (c) 2013-2020 FibreCode GmbH <info@fibrecode.com>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import binascii

from .eth_frame import EthFrame

class RxFrameTypes:
    RxFrameTypeData=1
    RxFrameTypeIdle=2
    RxFrameTypeTrigger=3


class RawRxFrame:
    """Raw-RxFrame class for Raw-Receive en/decoding"""
    def __init__(self, frameType, rawData):
        """Raw Frame as received from FC611/FC612/FC631 incl. timestamp, seq-counter"""
        self.FrameType = frameType
        if frameType == 1:
            if len(rawData) > 26:
                self.TimestampSec = int.from_bytes(rawData[0:8], byteorder='little')
                self.TimestampNano = int.from_bytes(rawData[8:12], byteorder='little')
                self.SequenceCnt = int.from_bytes(rawData[12:16], byteorder='little')
                self.EthFrame = EthFrame(rawData[16:])
                self.EthFullFrame = rawData[16:]
            else:
                self.TimestampSec = -1
                self.TimestampNano = -1
                self.SequenceCnt = -1
                self.EthFrame = None
                self.EthFullFrame = None
        elif frameType == 2:    # idle
            self.TimestampSec = int.from_bytes(rawData[0:8], byteorder='little')
            self.TimestampNano = int.from_bytes(rawData[8:12], byteorder='little')
            self.SequenceCnt = int.from_bytes(rawData[12:16], byteorder='little')
        elif frameType == 3:
            self.TimestampSec = int.from_bytes(rawData[0:8], byteorder='little')
            self.TimestampNano = int.from_bytes(rawData[8:12], byteorder='little')
            self.SequenceCnt = int.from_bytes(rawData[12:16], byteorder='little')
            self.EventNumber = int.from_bytes(rawData[16:24], byteorder='little')
            self.EventSource = int.from_bytes(rawData[24:25], byteorder='little')
            self.EventType = int.from_bytes(rawData[25:26], byteorder='little')

        else:
            self.FrameType = -1

    def __str__(self):
        
        try:
            return '<{0}:{1}.{2}:{3}>{4}'.format(self.FrameType, self.TimestampSec, self.TimestampNano, self.SequenceCnt, self.EthFrame)
        except:
            return 'N/A'

    def get_timestamp_ns(self):
        """get full ns of sec and nanos"""
        return self.TimestampSec * 1e9 + self.TimestampNano

    def get_eth_frame(self):
        return self.EthFullFrame



class RawTxFrame:
    """Raw-TxFrame class for Raw-Receive en/decoding"""
    def __init__(self, frameId, timestampSec, timestampNano, ethFrame):
        """Raw Frame after sent, optional incl. timestamp and frameId"""
        self.FrameId = frameId
        self.TimestampSec = timestampSec
        self.TimestampNano = timestampNano
        self.EthFrame = EthFrame(ethFrame)
        self.EthFullFrame = ethFrame

    def __str__(self):
        
        try:
            return '<{0}:{1}.{2}>{3}'.format(self.FrameId, self.TimestampSec, self.TimestampNano, self.EthFrame)
        except:
            return 'N/A'

    def get_timestamp_ns(self):
        """get full ns of sec and nanos"""
        return self.TimestampSec * 1e9 + self.TimestampNano

    def get_eth_frame(self):
        return self.EthFullFrame



